#include "times.h"
#include <gtk/gtk.h>
#include <glade/glade.h>
#include "eggtrayicon.h"

#define TRAYICON_IMAGE "/usr/share/icons/Bluecurve/16x16/apps/icon-email.png"

GladeXML* xml;
extern int dst;
/* extern int athan; */
extern int method;
extern int remind;
extern double gmtDiff;
extern double seaLevel;
extern double latitude;
extern double longitude;
extern char* locationName;

static void on_prefsDialog_response(GtkDialog* dialog, gint response){
   GtkWidget* entry, *widget, *button;
 
   widget = (GtkWidget*)dialog;
   gtk_widget_hide(widget);

   if (response == GTK_RESPONSE_OK){
      char* tmp;
      entry = glade_xml_get_widget(xml, "locName_entry");
      locationName = g_strdup(gtk_entry_get_text((GtkEntry*)entry));
      entry = glade_xml_get_widget(xml, "lat_entry");
      latitude = g_ascii_strtod(gtk_entry_get_text((GtkEntry*)entry), NULL);
      entry = glade_xml_get_widget(xml, "long_entry");
      longitude = g_ascii_strtod(gtk_entry_get_text((GtkEntry*)entry), NULL);
      entry = glade_xml_get_widget(xml, "gmtdiff_entry");
      gmtDiff = g_ascii_strtod(gtk_entry_get_text((GtkEntry*)entry), NULL);
      entry = glade_xml_get_widget(xml, "sealevel_entry");
      seaLevel = g_ascii_strtod(gtk_entry_get_text((GtkEntry*)entry), NULL);

      button = glade_xml_get_widget(xml, "dst_checkbox");
      dst = (gtk_toggle_button_get_active((GtkToggleButton*)button) == 
                                          TRUE)? 1 : 0;
      button = glade_xml_get_widget(xml, "mathhab");
      method = gtk_combo_box_get_active((GtkComboBox*)button);
      button = glade_xml_get_widget(xml, "reminderOptionBox");
      remind = (gtk_toggle_button_get_active((GtkToggleButton*)button) == 
                                             TRUE)? 1 : 0;
      /*
      button = glade_xml_get_widget(xml, "athanOptionBox");
      athan = (gtk_toggle_button_get_active((GtkToggleButton*)button) == 
                                             TRUE)? 1 : 0;
      */
      reCalculateTimes();
      saveSettings();
   }
}

static void prefs_dialog(){
   char* tmp;
   GtkWidget* widget, *entry, *button;
   widget = glade_xml_get_widget(xml, "prefsDialog");
   g_signal_connect(widget, "response",
                    G_CALLBACK(on_prefsDialog_response), NULL);

   entry = glade_xml_get_widget(xml, "locName_entry");
   gtk_entry_set_text((GtkEntry*)entry, locationName);
   entry = glade_xml_get_widget(xml, "lat_entry");
   asprintf(&tmp, "%f", latitude);
   gtk_entry_set_text((GtkEntry*)entry, tmp);
   free(tmp);
   asprintf(&tmp, "%f", longitude);
   entry = glade_xml_get_widget(xml, "long_entry");
   gtk_entry_set_text((GtkEntry*)entry, tmp);
   free(tmp);
   asprintf(&tmp, "%f", gmtDiff);
   entry = glade_xml_get_widget(xml, "gmtdiff_entry");
   gtk_entry_set_text((GtkEntry*)entry, tmp);
   free(tmp);
   asprintf(&tmp, "%f", seaLevel);
   entry = glade_xml_get_widget(xml, "sealevel_entry");
   gtk_entry_set_text((GtkEntry*)entry, tmp);
   free(tmp);
   
   button = glade_xml_get_widget(xml, "dst_checkbox");
   gtk_toggle_button_set_active((GtkToggleButton*)button, 
                                (dst==0)? FALSE : TRUE);

   button = glade_xml_get_widget(xml, "reminderOptionBox");
   gtk_toggle_button_set_active((GtkToggleButton*)button, 
                                (remind==0)? FALSE : TRUE);
   /*
   button = glade_xml_get_widget(xml, "athanOptionBox");
   gtk_toggle_button_set_active((GtkToggleButton*)button, 
                                (athan==0)? FALSE : TRUE);
   */
   button = glade_xml_get_widget(xml, "mathhab");
   gtk_combo_box_set_active((GtkComboBox*)button, method);
   gtk_widget_show(widget);
}

static void on_about_response(GtkDialog* dialog, gint response){
   gtk_widget_hide((GtkWidget*)dialog);
}

static void about_dialog(){
   GtkWidget* dialog;
   dialog = glade_xml_get_widget(xml, "aboutDialog");
   g_signal_connect(dialog, "response", 
                    G_CALLBACK(on_about_response), NULL);
   gtk_widget_show(dialog);
}

static void toggle_window(GdkEventButton* event){
   static int status = 1;
   
   GtkWidget* window;
   window = glade_xml_get_widget(xml, "mainWindow");
   if (status) gtk_widget_hide_all(window);
   else gtk_widget_show_all(window);

   status = status ^ 1;
}

static void show_menu(GdkEventButton* event){
   GtkWidget* menu, *item;

   menu = gtk_menu_new();
   item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
   g_signal_connect_swapped(item, "activate", G_CALLBACK(prefs_dialog), NULL);
   gtk_menu_attach(GTK_MENU(menu), item, 0, 1, 0, 1);
   gtk_widget_show(item);
   item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DIALOG_INFO, NULL);
   g_signal_connect_swapped(item, "activate", G_CALLBACK(about_dialog), NULL);
   gtk_widget_show(item);
   gtk_menu_attach(GTK_MENU(menu), item, 0, 1, 1, 2);
   item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
   g_signal_connect_swapped(item, "activate", G_CALLBACK(gtk_main_quit), NULL);
   gtk_menu_attach(GTK_MENU(menu), item, 0, 1, 2, 3);
   gtk_widget_show(item);
   gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 
                  event->button, event->time);
}

static void icon_clicked_cb(GtkWidget* button, 
                            GdkEventButton* event, 
                            gpointer data){
   if (event->type != GDK_BUTTON_PRESS)
      return;
   if (event->button == 1)
      toggle_window(event);
   else if (event->button == 3)
      show_menu(event);
}

gint main(gint argc, gchar** argv){
   GdkPixbuf* pixbuf;
   GtkWidget* box, *image;
   EggTrayIcon* tray_icon;

   gtk_init(&argc, &argv);
   xml = glade_xml_new("../namaztime.glade", NULL, NULL);
   if (!xml) g_error("unable to load namaztime.glade!");

   initTimes();
   glade_xml_signal_autoconnect(xml);

   tray_icon = egg_tray_icon_new("prayertimes");
   box = gtk_event_box_new();
   image = gtk_image_new();
   g_signal_connect(box, "button-press-event", 
                    G_CALLBACK(icon_clicked_cb), NULL);
   gtk_container_add(GTK_CONTAINER(box), image);
   gtk_container_add(GTK_CONTAINER(tray_icon), box);
   gtk_widget_show_all(GTK_WIDGET(tray_icon));
   g_object_ref(tray_icon);
   pixbuf = gdk_pixbuf_new_from_file(TRAYICON_IMAGE, NULL);
   gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
   g_object_unref(pixbuf);

   gtk_main();
   return 0;
}
