#include "times.h"
#include "prayer.h"
#include <gtk/gtk.h>

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char** labelText;
char* labelNames[] = { "frameLabel", "fajrLabel", "shuruqLabel", "zuhrLabel",
                        "asrLabel", "maghribLabel", "ishaLabel" };
char* lblLabels[] = { "Fajr", "Shuruq", "Dhuhr", "Asr", "Maghrib", "Isha" };

int dst;
/* int athan; */
int method;
int remind;
double gmtDiff;
double seaLevel;
double latitude;
double longitude;
char* locationName;

int nextsalat;
Prayer ptList[6];

void restoreSettings(){
   FILE* fp;
   char str[80];
   char* filestr;

   asprintf(&filestr, "%s%s", getenv("HOME"), "/.praytime_config");
   fp = fopen(filestr, "r");
   free(filestr);

   if (fp == NULL){
      latitude = 24.4833;
      longitude = 54.35;
      method = 7;
      gmtDiff = 4;
      dst = 0;
      remind = 0;
      /* athan = 0; */
      locationName = strdup("Abu Dhabi");
      return;
   }

   while (fgets(str, 80, fp)!=NULL){
      char* tmp = strchr(str, ' ');
      if (!tmp) continue;
      tmp++;
      
      if (strstr(str, "location: ")!=NULL){
         locationName = strdup(tmp);
         tmp = strrchr(locationName, '\n');
         if (tmp != NULL) *tmp = '\0';
      }
      else if (strstr(str, "latitude: ")!=NULL)
         latitude = strtod(tmp, NULL);
      else if (strstr(str, "longitude: ")!=NULL)
         longitude = strtod(tmp, NULL);
      else if (strstr(str, "gmtDiff: ")!=NULL)
         gmtDiff = strtod(tmp, NULL);
      else if (strstr(str, "seaLevel: ")!=NULL)
         seaLevel = strtod(tmp, NULL);
      else if (strstr(str, "dst: ")!=NULL)
         dst = atoi(tmp);
      else if (strstr(str, "method: ")!=NULL)
         method = atoi(tmp);
      else if (strstr(str, "remind: ")!=NULL)
         remind = atoi(tmp);
      /*
      else if (strstr(str, "athan: ")!=NULL)
         athan = atoi(tmp);
      */
   }
}

void saveSettings(){
   FILE* fp;
   char* outfile;

   asprintf(&outfile, "%s%s", getenv("HOME"), "/.praytime_config");
   fp = fopen(outfile, "w");
   free(outfile);

   fprintf(fp, "location: %s\n", locationName);
   fprintf(fp, "latitude: %f\n", latitude);
   fprintf(fp, "longitude: %f\n", longitude);
   fprintf(fp, "gmtDiff: %f\n", gmtDiff);
   fprintf(fp, "seaLevel: %f\n", seaLevel);
   fprintf(fp, "dst: %d\n", dst);
   fprintf(fp, "method: %d\n", method);
   fprintf(fp, "remind: %d\n", remind);
   /* fprintf(fp, "athan: %d\n", athan); */
   fprintf(fp, "###");
   fclose(fp);
}

void findSalatTimes(){
   int i, val;
   Date date;
   Method conf;
   Location loc;
   time_t curTime;
   struct tm* timeStruct;
   static int initialized = 0;
   
   loc.degreeLat = latitude;
   loc.degreeLong = longitude;

   loc.dst = dst;
   loc.gmtDiff = gmtDiff;

   loc.seaLevel = seaLevel;
   loc.pressure = 1010;
   loc.temperature = 10;

   curTime = time(NULL);
   timeStruct = localtime(&curTime);
   date.day = timeStruct->tm_mday;
   date.month = timeStruct->tm_mon + 1;
   date.year = timeStruct->tm_year + 1900;

   getMethod(method, &conf);
   conf.round = 0;
   getPrayerTimes(&loc, &conf, &date, ptList);

   if (initialized == 0){
      labelText = malloc(7 * sizeof(char*));
      initialized++;
   }
   else {
      for (i=1; i<7; i++)
         free(labelText[i]);
   }

   labelText[0] = locationName;

   val = -1;
   for (i=0; i<6; i++){
      if (i==1) continue;
      if ((timeStruct->tm_hour < ptList[i].hour) ||
          ((timeStruct->tm_hour == ptList[i].hour) &&
           (timeStruct->tm_min < ptList[i].minute))){
         val = i;
         break;
      }
   }

   if (val == -1) val = 0;
   
   for (i=0; i<6; i++)
      asprintf(&(labelText[i+1]), 
         "<span foreground=\"%s\"><b>%s:</b></span> %3d:%02d:%02d%s", 
         (i==val)? "red" : "black", lblLabels[i],
         (ptList[i].hour > 12)? ptList[i].hour-12 :
         ptList[i].hour, ptList[i].minute, 
         ptList[i].second, (ptList[i].hour>=12)?
         " pm" : " am");
   nextsalat = val;
}

static void on_salatTime_response(GtkDialog* dialog, gint response){
   gtk_widget_hide((GtkWidget*)dialog);
}

/* TODO: add athan functionality
void playAthan(){}
*/

int checkSalatTime(gpointer data){
   time_t curTime;
   struct tm* timeStruct;

   curTime = time(NULL);
   timeStruct = localtime(&curTime);
   if ((ptList[nextsalat].hour == timeStruct->tm_hour) &&
       (ptList[nextsalat].minute < timeStruct->tm_min)){
      if (remind == 1){
         GtkWidget* dialog;
         dialog = glade_xml_get_widget(xml, "salatTimeDialog");
         g_signal_connect(dialog, "response",
                          G_CALLBACK(on_salatTime_response), NULL);
         gtk_widget_show(dialog);
      }
      /*
      if (athan == 1)
         playAthan();
      */
      reCalculateTimes();
   }
   return TRUE;
}

void initTimes(){
   restoreSettings();
   reCalculateTimes();
   g_timeout_add(1000 * 60, checkSalatTime, NULL);
}

void reCalculateTimes(){
   int i;
   GtkWidget* label;

   findSalatTimes();

   for (i=0; i<7; i++){
      label = glade_xml_get_widget(xml, labelNames[i]);
      gtk_label_set_text((GtkLabel*)label, labelText[i]);
      gtk_label_set_use_markup((GtkLabel*)label, TRUE);
   }
}
